require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/company/company_create'
require 'uu/os/company/company_attributes'
require 'uu/os/company/company_entry_attributes'
require 'uu/os/company/company_set_attributes'
require 'uu/os/company/company_get_entry_list'
require 'uu/os/rest/paging_criteria'

module UU
  module OS
    # Company service.
    # @deprecated Very soon, it will not be possible to create artifacts of this class
    module Company
      extend self

      # Service path
      PATH = 'ues/core/container/UESCompany'
      # Creates a new company. The command creates a new company in a specified location.
      # At the very least meta artifact must be specified in {CompanyCreate}
      # Object. Competent role for the company will be selected as the most suitable according
      # to specified container and meta artifact (executive/authorized role with connected
      # interface), or can be also specified in {CompanyCreate} Object. 
      #
      # @param [String, UU::OS::UESURI] location_uri UESURI of the container except meta model where the company is going to be created 
      # @param [CompanyCreate] company Object containing attributes of new company 
      #
      # @return [UU::OS::UESURI] UESURI of the new company 
      def create(location_uri, company = nil)
        svc = UU::OS::REST::RemoteClient.new(Company)
        payload = UU::OS::Company::CompanyCreate.new(company).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # This command returns attributes representing the company specified by the companyUri
      # parameter. The command does not change the state of the company in the system. 
      #
      # @param [String, UU::OS::UESURI] company_uri UESURI of the company whose attributes are returned 
      #
      # @return [CompanyAttributes] Filled Object with attributes of the company 
      def get_attributes(company_uri)
        svc = UU::OS::REST::RemoteClient.new(Company)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', company_uri)
          return Company::CompanyAttributes.new(res)
        end
      end

      # Command for setting attributes of a company. The command cannot modify references
      # to objects like location, competent role and so on. 
      #
      # @param [String, UU::OS::UESURI] company_uri UESURI of the company whose attributes are to be changed. 
      # @param [CompanySetAttributes] company Object containing new attributes for company 
      #
      # @return [UU::OS::UESURI] UESURI of the company 
      def set_attributes(company_uri, company = nil)
        svc = UU::OS::REST::RemoteClient.new(Company)
        payload = UU::OS::Company::CompanySetAttributes.new(company).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', company_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # This command deletes the specified company from the system. The company is deleted
      # even if constains locked sheet or attachment. The company cannot be deleted when:
      #   the company is not empty. 
      #   the company's workflow is not created 
      #   a property contains a reference to this company 
      #   a time sheet exists with a reference to this company 
      #
      # @param [String, UU::OS::UESURI] company_uri UESURI of the company which is going to be deleted
      def delete(company_uri)
        svc = UU::OS::REST::RemoteClient.new(Company)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('delete', company_uri)
        end
      end

      # This command returns a list of entries (artifacts or shortcuts) enlisted in the
      # specified company. Only entries listed exactly in the specified company are considered
      # (so subordinate organizational units are not considered). The command does not
      # return all entries, only a sublist. The list of returned entries could be filtered
      # by the name, code or entityType of an entry. The list is sorted by the name (and codes
      # in case are equal) of a entry by default, or it can be sorted arbitrarily by any of
      # entry attributes usable for the filtering or these attributes combinations.
      # See the UES query documentation for more information. 
      #
      # @param [String, UU::OS::UESURI] company_uri UESURI of the company from where entries are listed 
      # @param [CompanyGetEntryList] criteria Object containing criteria for listing Available parameters are: 
      # {CompanyEntryAttributes#name name}, {CompanyEntryAttributes#code code} and 
      # {CompanyEntryAttributes#entityTypeUri entityTypeUri}. 
      #
      # See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      #
      # @return [UU::OS::REST::ResultList<Company::CompanyGetEntryList, Company::CompanyEntryAttributes>] List of Objects representing company entries 
      def get_entry_list(company_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(Company)
        dto = Company::CompanyGetEntryList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        svc.add_parameter('recursive', dto.recursive)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getEntryList', company_uri)
          return UU::OS::REST::ResultList.new(Company::CompanyGetEntryList, Company::CompanyEntryAttributes, res)
        end
      end

    end
  end
end
