require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/role/role_create'
require 'uu/os/role/role_attributes'
require 'uu/os/role/role_set_attributes'
require 'uu/os/role/role_get_artifact_list'
require 'uu/os/role/role_get_artifact_list_item'
require 'uu/os/role/role_add_role_interface'
require 'uu/os/role/role_remove_role_interface'
require 'uu/os/role/role_get_role_interface_list'
require 'uu/os/role/role_get_role_interface_list_item'

module UU
  module OS
    module Role
      extend self

      # Service path
      PATH = 'ues/core/role/UESRole'

      # Creates a new role. The command creates a new role in a specified location.At the
      # very least meta artifact must be specified in {RoleCreate} DTO. Competent
      # role for the new role will be selected as the most suitable according to specified
      # container and meta artifact (executive/authorized role with connected interface),
      # or can be also specified in {RoleCreate} DTO.
      #
      # @param [String, UU::OS::UESURI] location_uri UESURI of the folder/org. unit/meta model where the role will be created.
      # @param [UU::OS::Role::RoleCreate] role Specific options.
      # @option role [String, UU::OS::UESURI] :meta_artifact_uri UESURI of the meta artifact. Only a meta artifact with the template class of Role (not its descendants) can be used. If the rights does not permit to create a new role from the meta artifact in the specified location, the create command fails.
      # @option role [String] :code Code of the new role. If nil is set, the code is generated automatically using the system sequence. The code must be unique in the given territory, otherwise the create command fails.
      # @option role [String] :name Name of the new role. If nil is set, tName from meta artifact is used if set, otherwise the create command fails. Note that the role name is automatically extended by the name of casted access roles names. Therefore the name attribute returned by getAttributes command returns a different string.
      # @option role [String] :description Description of the new role. If nil is set, tDescription from meta artifact is used if set, otherwise no description is set on the new role.
      # @option role [String, UU::OS::UESURI] :competent_role_uri UESURI of the competent role. If nil is set, the executive role from authorization is used. If no role is authorized to create the role into specified container, the create command fails. If the selected role doesn't have connected the role interface with the selected meta artifact, the create command fails. If the selected role doesn't belong to the same organizational unit as a new role or if it doesn't belong to one of superior organizational units,the create command fails.
      # @option role [String, UU::OS::Artifact::SecurityLevel] :security_level Security level. If nil is set, security level is taken from the meta artifact tSecurityLevel. If the meta artifact tSecurityLevel is higher than the user's security clearance, the create command fails.
      # @option role [Boolean] :searchable_in_territory Specifies if it is possible to search the new role in its territory. If attribute is nil, value will be set according to the value of meta artifact (The create command fails if the value also is nil).
      # @option role [Boolean] :searchable_across_territories <b>Deprecated.</b> Specifies if it is possible to search and cast the new role in other territories. If attribute is nil, value will be set according to the value of meta artifact (The create command fails if the value also is nil). This attribute has been deprecated and will not be supported in the future.
      # @option role [Boolean] :territory_switch_visible <b>Deprecated.</b> Specifies if the organizational unit of this role should be visible in visual usecase for displaying list of territories and organizational units. If true is set then related organizational unit is displayed. If false is set then related organizational unit is not visible unless there is another role with this flag set to true. If nil is set, tTerritorySwitchVisible from meta artifact is used if set, otherwise the create command fails. This attribute has been deprecated. The visual use case for displaying territories and organizational units, which this attribute was used for, is no longer supported.
      # @option role [String, UU::OS::UESURI] :icon_uri URI of the role's icon. If nil is set, the icon is not modified. If nil URI ("ues:{[-1]}:{[-1]}:") is set, the role icon is removed and the role uses icon from meta artifact. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option role [Boolean] :grant_all_from_organizational_unit <b>Deprecated.</b> Specifies if the role will be included among All roles from organizational unit. If true or nil is set, the role will be included,otherwise it will not be included among All roles from organizational unit.
      # @option role [Boolean] :dry_run Specifies whether this is only a test run to verify that the object can be created. The default value is false.
      # @option role [Boolean] :component_content Defines whether the primarily displayed content of the created Role is component content. Default value is false.
      #
      # @return [UU::OS::UESURI] UESURI of the new role
      #
      # @example
      #   # Create new role.
      #   UU::OS::Role.create('ues:TERRITORY:FOLDER', :code => 'NEW_ROLE', :name => 'New Role')
      #
      #   # Verifying that new role can be created.
      #   UU::OS::Role.create('ues:TERRITORY:FOLDER', :code => 'NEW_ROLE', :name => 'New Role', :dry_run => true)
      #
      def create(location_uri, role = nil)
        svc = UU::OS::REST::RemoteClient.new(Role)
        payload = UU::OS::Role::RoleCreate.new(role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, payload)
          # res is nil if dry_run was set to true. In that case, return nil (do nothing)
          if !(res.nil? || res.empty?)
            return UU::OS::UESURI.new(res)
          end
        end
      end

      # Returns attributes representing the role specified by theroleUri
      # parameter. The command does not change the state of the role in the system.
      #
      # @param [String, UU::OS::UESURI] role_uri UESURI of the role
      #
      # @return [RoleAttributes] DTO with role attributes.
      # @example
      #   # Get role attributes
      #   UU::OS::Role.get_attributes('ues:TERRITORY:ROLE')
      #
      def get_attributes(role_uri)
        svc = UU::OS::REST::RemoteClient.new(Role)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', role_uri)
          return Role::RoleAttributes.new(res)
        end
      end

      # Command for setting attributes of a role. The command can't modify references
      # to objects like location, competent role and so on.
      #
      # @param [String, UU::OS::UESURI] role_uri UESURI of the role whose attributes are to be changed
      # @param [RoleSetAttributes] role DTO containing new attributes for the role
      # @option role [String] :name The new name of the role. If nil is set, the name is not modified.<br>Note that the role name is automatically extended by the name of casted access roles names, if this attribute is not nil. Therefore the name attribute returned by the getAttributescommand contains different string.
      # @option role [String] :code The new code of the role. If nil is set, the code is not modified.The code must be unique in a given territory, otherwise thesetAttributes command fails.
      # @option role [String] :description The new description of the role. If nil is set, the description is not modified.
      # @option role [String, UU::OS::Artifact::SecurityLevel] :security_level The new security level of the role. If nil is set, the security level is not modified. If the meta artifact tSecurityLevel is higher than the user's security clearance, the setAttributescommand fails.
      # @option role [Boolean] :searchable_in_territory Specifies if it is possible to search the role in its territory. If nil is set, this attribute on the role is not modified.
      # @option role [Boolean] :searchable_across_territories <b>Deprecated.</b> Specifies if it is possible to search and cast the role in other territories. If nil is set, this attribute on the role is not modified.
      # @option role [Boolean] :territory_switch_visible <b>Deprecated.</b> Specifies if the organizational unit of this role should be visible in visual usecase for displaying list of territories and organizational units. If true is set then related organizational unit is displayed. If false is set then related organizational unit is not visible unless there is another role with this flag set to true.
      # @option role [String, UU::OS::UESURI] :icon_uri URI of the role's icon. If nil is set, the icon is not modified. If nil UESURI ("ues:{[-1]}:{[-1]}:") is set, the role icon is removed and the role uses icon from meta artifact. Allowed format of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}'. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option role [Boolean] :grant_all_from_organizational_unit <b>Deprecated.</b> Specifies if the role will be included among All roles from organizational unit. If nil is set, this attribute on the role is not modified.
      # @option role [Boolean] :component_content Defines whether the primarily displayed content of the created Role is component content. Default value is false.
      #
      # @return [UU::OS::UESURI] UESURI of the role
      #
      # @example
      #   # Set attributes of role - make it non searchable in territory.
      #   UU::OS::Role.set_attributes('ues:TERRITORY:ROLE', :name=>'John', :code=>'NEWCODE', :description=>'Some sample description', :searchable_in_territory=>false)
      def set_attributes(role_uri, role = nil)
        svc = UU::OS::REST::RemoteClient.new(Role)
        payload = UU::OS::Role::RoleSetAttributes.new(role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', role_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Deletes the specified role from the system. The role is deleted even
      # if it contains a locked sheet or attachment. The role cannot be deleted if:
      #   the role is not in an active cast
      #   the role's workflow is not created
      #   a property contains a reference to this role
      #   a time sheet exists with a reference to this role
      #   an artifact exists with this role as competent role
      #   an activity exists with this role as competent role
      #
      # @param [String, UU::OS::UESURI] role_uri UESURI of the role which is about to be deleted.
      #
      # @example
      #   # Delete role specified by UESURI.
      #   UU::OS::Role.delete('ues:TERRITORY:DELETE')
      #
      def delete(role_uri)
        svc = UU::OS::REST::RemoteClient.new(Role)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('delete', role_uri)
        end
      end

      # Returns a list of entries representing the artifacts for that the
      # specified role is competent. The list is sorted by the name(and codes in case are
      # equal) of a entry by default.
      #
      # @param [String, UU::OS::UESURI] role_uri UESURI of the role that is competent for the searched artifacts
      # @param [RoleGetArtifactList] criteria DTO containing criteria for listing role'sartifacts.
      #
      # @return [UU::OS::REST::ResultList<Role::RoleGetArtifactList, Role::RoleGetArtifactListItem>] List of DTO representing artifacts for that the specified role is competent.
      #
      # @example
      #   # Get role artifact list.
      #   UU::OS::Role.get_artifact_list('ues:TERRITORY:ROLE')
      #
      def get_artifact_list(role_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(Role)
        dto = Role::RoleGetArtifactList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getArtifactList', role_uri)
          return UU::OS::REST::ResultList.new(Role::RoleGetArtifactList, Role::RoleGetArtifactListItem, res)
        end
      end

      # Creates new role interface connection to the specified role. The
      # role interface has to be in an active state and mustn't be already connected. The
      # role interface has to be connected to the one of the authorized roles for this command,
      # or the authorized role has to be casted to the default group of meta model where the
      # role interface is located.
      #
      # @param [String, UU::OS::UESURI] role_uri UESURI of the role where the role interface will be connected
      # @param [RoleAddRoleInterface] role DTO containing role interface that to be connected
      # @option role [String, UU::OS::UESURI] :role_interface_uri UESURI of the role interface that to be connected to the specified role. If nil is set, the command fails.
      #
      # @example
      #   # Add role interface to specified role.
      #   UU::OS::Role.add_role_interface('ues:TERRITORY:ROLE', :role_interface_uri=>'ues:TERRITORY:ROLE_ITERFACE')
      #
      def add_role_interface(role_uri, role = nil)
        svc = UU::OS::REST::RemoteClient.new(Role)
        payload = UU::OS::Role::RoleAddRoleInterface.new(role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('addRoleInterface', role_uri, payload)
        end
      end

      # Removes role interface connection from the specified role.Only connected
      # interfaces can be disconnected from role.
      #
      # @param [String, UU::OS::UESURI] role_uri UESURI of the role where the role interface will be disconnected.
      # @param [RoleRemoveRoleInterface] role DTO containing role interface that to be disconnected
      # @option role [String, UU::OS::UESURI] :role_interface_uri UESURI of the role interface that to be disconnected from the specified role. If nil is set, the command fails.
      #
      # @example
      #   # Remove role interface for specified role.
      #   UU::OS::Role.remove_role_interface('ues:TERRITORY:ROLE', :role_interface_uri=>'ues:TERRITORY:ROLE_IFC')
      #
      def remove_role_interface(role_uri, role = nil)
        svc = UU::OS::REST::RemoteClient.new(Role)
        payload = UU::OS::Role::RoleRemoveRoleInterface.new(role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('removeRoleInterface', role_uri, payload)
        end
      end

      # Returns list of the role interfaces that are connected to the specified role. Returned list does not contain any objects filtered out by executed UESQuery.
      #
      # @param [String, UU::OS::UESURI] role_uri UESURI of the role for which the connected role interfaces are to be listed.
      # @param [RoleGetRoleInterfaceList] criteria DTOcontaining criteria for listing connected role interfaces. See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      # @option criteria [String] :query # The <b>UES query</b> string. It has to conform the UESQ syntax (see documentation for more info). Filtering and order of the result list can be specified there. If nil is set, no filtering is applied and entries are ordered by name of casted subjects (and code as the second criteria).
      #
      # @return [UU::OS::REST::ResultList<Role::RoleGetRoleInterfaceList, Role::RoleGetRoleInterfaceListItem>] List of DTO representing role interfaces which are connected to the specified role.
      #
      # @example
      #   # Get role interface list descend ordered by code.
      #   roleIfcList = UU::OS::Role.get_role_interface_list('ues:TERRITORY:ACCESSROLE', :query=>'ORDER BY code DESC')
      #
      def get_role_interface_list(role_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(Role)
        dto = Role::RoleGetRoleInterfaceList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getRoleInterfaceList', role_uri)
          return UU::OS::REST::ResultList.new(Role::RoleGetRoleInterfaceList, Role::RoleGetRoleInterfaceListItem, res)
        end
      end

    end
  end

end
