require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/shortcut/shortcut_create'
require 'uu/os/shortcut/shortcut_attributes'
require 'uu/os/shortcut/shortcut_get_shortcut_list'
require 'uu/os/shortcut/shortcut_relocate'
require 'uu/os/shortcut/shortcut_set_attributes'

module UU
  module OS
    module Shortcut
      extend self

      # Service path
      PATH = 'ues/core/container/UESShortcut'
      # This command creates a new shortcut for the specified artifact. At the very least,
      # referenced artifact have to be specified in this command.
      #
      # @param [String, UU::OS::UESURI] location_uri UESURI of the location (folder, org. unit, territory etc.) where the new shortcut is created
      # @param [ShortcutCreate] shortcut DTO containing attributes of the new shortcut
      # @option shortcut [String] :name Name of the new shortcut. When nil is set, the name of the referenced artifact is used (and this equality is kept from now on for a shortcut referencing an artifact in the same territory).
      # @option shortcut [String] :code Code of the new shortcut. When nil is set, the code is generated automatically using the system sequence. The code must be unique in a given territory, otherwise the create command fails.
      # @option shortcut [String, UU::OS::UESURI] :artifact_uri UESURI of the referenced artifact. Cannot be nil.
      #
      # @return [UU::OS::UESURI] UESURI of the created shortcut.
      #
      # @example
      #   # Create shortcut in specified location
      #   UU::OS::Shortcut.create('ues:TERRITORY:LOCATION', :name=>'Shortcut name', :code=>'CODE', :artifact_uri=>'ues:TERRITORY:ARTIFACT')
      def create(location_uri, shortcut = nil)
        svc = UU::OS::REST::RemoteClient.new(Shortcut)
        payload = UU::OS::Shortcut::ShortcutCreate.new(shortcut).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # This command returns attributes representing the shortcut specified by the shortcutUri
      # parameter. The command does not change the state of the shortcut in the system.
      #
      # @param [String, UU::OS::UESURI] shortcut_uri UESURI of a shortcut whose attributes are returned
      #
      # @return [ShortcutAttributes] DTO containing the shortcut attributes
      #
      # @example
      #   # Get shortcut attributes.
      #   UU::OS::Shortcut.get_attributes('ues:TERRITORY:SHORTCUT')
      #
      def get_attributes(shortcut_uri)
        svc = UU::OS::REST::RemoteClient.new(Shortcut)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', shortcut_uri)
          return Shortcut::ShortcutAttributes.new(res)
        end
      end

      # This command returns a list of shortcuts to the specified artifact. The command
      # does not return all shortcuts, only a sublist. Only the shortcuts from the territory
      # of the artifact are returned. The list of returned shortcuts could be filtered
      # by the name or code of a shortcut. The list is sorted by the name (and codes in case
      # are equal) of a shortcut by default, or it can be sorted arbitrarily by any of shortcut
      # attributes usable for the filtering or these attributes combinations. See the
      # UES query documentation for more information.
      #
      # @param [String, UU::OS::UESURI] artifact_uri UESURI of the artifact to which the returned shortcuts point.
      # @param [ShortcutGetShortcutList] criteria DTO containing criteria for listing shortcuts. Available parameters are: {ShortcutAttributes#name name} and {ShortcutAttributes#code code}. See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      # @option criteria [String] :query The <b>UES query</b> string. It has to conform the ues query syntax (see documentation for more info). Filtering and order of the result list can be specified there. When nil is set, no filtering is applied and entries are ordered by name and code as the second criteria.
      # @option criteria [String] :code_filter Filter for filtering the result according to the shortcut code. When nil is set or if codeFilter is an empty string, no filtering according to the code is applied. Filter behavior is similar to the SQL like predicate pattern string. It is a wildcard with these special characters: % (percent sign) allows matching any string of any length (including zero length) _ (underscore) allows matching a single character \ (backslash) is used to escape any of these special characters. 
      # @option criteria [String] :name_filter Filter for filtering the result according to the shortcut name. When nil is set or if nameFilter is an empty string, no filtering according to the name is applied. Filter behavior is similar to the SQL like predicate pattern string. It is a wildcard with these special characters: % (percent sign) allows matching any string of any length (including zero length) _ (underscore) allows matching a single character \ (backslash) is used to escape any of these special characters. 
      # @return [UU::OS::REST::ResultList<Shortcut::ShortcutGetShortcutList, Shortcut::ShortcutAttributes>] List of DTO representing shortcuts. The same DTO as in the command getAttributes.
      #
      # @example
      #   # Get shortcut list with ordering.
      #   UU::OS::Shortcut.get_shortcut_list('ues:TERRITORY:ARTIFACT',:query=>'ORDER BY name, code')
      #
      def get_shortcut_list(artifact_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(Shortcut)
        dto = Shortcut::ShortcutGetShortcutList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getShortcutList', artifact_uri)
          return UU::OS::REST::ResultList.new(Shortcut::ShortcutGetShortcutList, Shortcut::ShortcutAttributes, res)
        end
      end

      # This command relocates the shortcut to the new location. It means the shortcut
      # is deleted and created again in a new location. It is recreated with the same attributes.
      # But its UESURI is different because the shortcut is deleted and created again in
      # another location. At the very least new location have to be specified in this command.
      # The new location has to be in the same territory.
      #
      # @param [String, UU::OS::UESURI] shortcut_uri UESURI of the shortcut to relocate
      # @param [ShortcutRelocate] shortcut DTO containing attributes for the shortcut relocation
      # @option shortcut [String, UU::OS::UESURI] :location_uri The new location for the shortcut. Cannot be nil.
      #
      # @return [UU::OS::UESURI] UESURI of the relocated shortcut. UESURI is different because the shortcut is deleted and created again in another location.
      #
      # @example
      #   # Relocate shortcut.
      #   UU::OS::Shortcut.relocate('ues:TERRITORY:SHORTCUT', :location_uri=>'ues:TERRITORY:LOCATION')
      #
      def relocate(shortcut_uri, shortcut = nil)
        svc = UU::OS::REST::RemoteClient.new(Shortcut)
        payload = UU::OS::Shortcut::ShortcutRelocate.new(shortcut).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('relocate', shortcut_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # This command deletes the shortcut from the system.
      #
      # @param [String, UU::OS::UESURI] shortcut_uri UESURI of the shortcut to be deleted
      #
      # @example
      #   # Delete shortcut specified by UESURI.
      #   UU::OS::Shortcut.delete('ues:TERRITORY:SHORTCUT')
      def delete(shortcut_uri)
        svc = UU::OS::REST::RemoteClient.new(Shortcut)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('delete', shortcut_uri)
        end
      end

      # This command updates some basic attributes of a shortcut.
      #
      # @param [String, UU::OS::UESURI] shortcut_uri UESURI of the shortcut whose attributes are to be changed.
      # @param [ShortcutSetAttributes] shortcut DTO containing new attributes the shortcut.
      # @option shortcut [String] :name The new name of the shortcut. When nil is set, the name is not modified. When empty string is set, the name is set the same as the name of referenced artifact (and this equality is kept from now on for a shortcut referencing an artifact in the same territory).
      # @option shortcut [String] :code The new code of the shortcut. When nil is set, the code is not modified. The code must be unique in a given artifact, otherwise the setAttributes command fails.
      # @option shortcut [String, UU::OS::UESURI] :artifact_uri UESURI of the referenced artifact. When nil is set, the reference is not modified.
      #
      # @return [UU::OS::UESURI] UESURI of the shortcut. It could be changed because command could change the code.
      #
      # @example
      #   # Set attributes for shortcut.
      #   UU::OS::Shortcut.set_attributes('ues:TERRITORY:SHORTCUT', :name=>'Shortcut name', :code=>'CODE', :artifact_uri=>'ues:TERRITORY:ARTIFACT')
      #
      def set_attributes(shortcut_uri, shortcut = nil)
        svc = UU::OS::REST::RemoteClient.new(Shortcut)
        payload = UU::OS::Shortcut::ShortcutSetAttributes.new(shortcut).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', shortcut_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

    end
  end

end
